package org.vishia.exampleJava2C.java4c;

import org.vishia.byteData.ByteDataAccessBase;

/**This class contains some variables, which are sent to a visualization app, 
 * an "Operation and Monitoring" app (OaM).
 * 
 * @author Hartmut Schorrig
 *
 */
public class OamVariables
{
  /**The timeStamp of the values in absolute milliseconds. */
  public long time;
  
  /**The current way, the measurement value. 1 um-unit. */
  public int way;
  
  /**The current way, set value. 100 um-unit*/
  public short wway;
  
  /**The target way, 100 um-unit. */
  public short targetWay;
  
  /**The abbreviation of the way-controller. 1 m-unit */ 
  public short dway;
  
  /**The output value of the controller. */
  public short output;
  
  public short stateSetValueGen;
  
  public byte ctController, ctSetValue;
  
  /**This class describes the struct of data in payload of an information block.
   * The position of values are given as byte position {@link #k_way} etc.
   * This class extends the ByteDataAccess. 
   * 
   * Note that the counterpart in the OaM 'ViewCfg.java' is a script file 'oamVar.cfg'
   * which names this variables and positions manually. 
   * It is because that application is only script controlled, not compiled with this source.
   * The script should be matching to this positions.
   */
  public static class OamVariablesByteAccess extends ByteDataAccessBase
  {
  	public final static int k_Time = 0,
  	                  k_way = 8,
  	                  k_wway=12,
  	                  k_targetWay=14,
  	                  k_dway = 16,
  	                  k_output = 18,
  	                  k_stateSetValueGen = 20,
  	                  k_ctController = 22,
  	                  k_ctSetValue = 23,
  	                  sizeofHead = 24;
  	

  	public OamVariablesByteAccess(){ super(sizeofHead); }
  	
    /**Write data to binary struct.
     * It adds only a payload with this data.
     * @param parent The descriptor for the binary struct of the information block 
     *   inside the whole telegram
     * @param data source
     */
		public void addToAndSetBinData(ByteDataAccessBase parent, OamVariables data)
		{
			parent.addChild(this);
			_setLong(k_Time, 8, data.time);
			setInt32(k_way, data.way);
      setInt16(k_wway, data.wway);
			setInt16(k_targetWay, data.targetWay);
			setInt16(k_dway, data.dway);
			setInt16(k_output, data.output);
			setInt16(k_stateSetValueGen, data.stateSetValueGen);
			setInt8(k_ctController, data.ctController);
			setInt8(k_ctSetValue, data.ctSetValue);
	  }
		
		/**Read data from binary struct
		 * @param parent The descriptor for the binary struct of the information block 
		 *   inside the whole telegram
		 * @param data destination
		 */
		public void addToAndGetBinData(ByteDataAccessBase parent, OamVariables data)
		{
			parent.addChild(this);
			data.time = _getLong(k_Time, 8);
      data.way = getInt32(k_way);
			data.wway = getInt16(k_wway);
			data.targetWay = getInt16(k_targetWay);
			data.dway = getInt16(k_dway);
			data.output = getInt16(k_output);
			data.stateSetValueGen = getInt16(k_stateSetValueGen);
			data.ctController = getInt8(k_ctController);
			data.ctSetValue = getInt8(k_ctSetValue);
	  }
		
  }
  
  
  
}
